#
# This file is part of Config-Model
#
# This software is Copyright (c) 2005-2017 by Dominique Dumont.
#
# This is free software, licensed under:
#
#   The GNU Lesser General Public License, Version 2.1, February 1999
#
[
  {
    'author' => [
      'Dominique Dumont'
    ],
    'class_description' => 'Options for special file system like \'bind\'',
    'copyright' => [
      '2010,2011 Dominique Dumont'
    ],
    'element' => [
      'bind',
      {
        'type' => 'leaf',
        'value_type' => 'boolean'
      }
    ],
    'license' => 'LGPL2',
    'name' => 'Fstab::NoneOptions'
  }
]
;

